//  Copyright 1998-1999 by Donn Cave, Seattle, Washington, USA.
//  All rights reserved.  Permission to copy, modify and distribute this
//  material is hereby granted, without fee, provided that the above
//  copyright notice appear in all copies.

//  Support functions for cross-module inheritance of ordinary class
//  methods and attributes.
//

#include <stdio.h>
#include <string.h>

#include "Python.h"

#include "PyBase.h"

PyObject *
Base_getattr(BaseObject *obj, char *name)
{
    //  Get attributes for object, using routine from each object type
    //  in list of base classes;  order presumably is up hierarchy.
    //  Each such object specifies itself as the first base class.
    //
    PyObject *ret, *list;
    ret = list = 0;

    //  For internal use, to recognize Bethon modules.
    if (!strcmp(name, BEBOSTAMP)) {
	Py_INCREF(Py_None);
	return Py_None;
    }
    //  If a list will be needed to accumulate all members, create it now.
    //  The simpleattr() functions will notice a non-null list parameter
    //  and infer __members__.
    //
    if (!strcmp(name, "__members__")) {
	ret = list = PyList_New(0);
	if (!list)
	    return 0;
    }

    int i;
    for (i = 0; i < MAXBASETYPES && obj->base[i].name[0]; ++i) {
	if (i > 0)
	    PyErr_Clear();
	ret = obj->base[i].simpleattr(obj, name, list);
	if (ret && !list)
	    break;
    }

    return ret;
}

void
Base_init(struct baseItem *base, const char *name, int offset, simpleattrfunc simple)
{
    int i;

    if (simple) {
	//  Find first empty slot.
	//
	for (i = 0; i < MAXBASETYPES && base[i].name[0]; ++i)
	    ;
	if (i < MAXBASETYPES) {
	    int len = strlen(name);
	    struct baseItem *ibase = &(base[i]);
	    if (len >= sizeof(ibase->name))
		len = sizeof(ibase->name) - 1;
	    memcpy(ibase->name, name, len);
	    ibase->name[len] = 0;
	    ibase->offset = offset;
	    ibase->simpleattr = simple;
	} else
	    fprintf(stderr, "\"%s\": Base limit exceeded.\n", name);
    } else {
	for (i = 0; i < MAXBASETYPES && base[i].name[0]; ++i) {
	    if (strcmp(base[i].name, name) == 0) {
		base[i].offset = offset;
		return;
	    }
	}
	fprintf(stderr, "\"%s\": Base not found in 2nd pass!\n", name);
    }
}

int
Base_initFromName(struct baseItem *base, const char *name, int offset)
{
    //  Get module and load base support.
    //
    char simple[128], *mod;
    struct BaseFunctions *basefun;
    if (PyErr_Occurred()) {
	// Sloppy bind attribute lookup, etc.
	fprintf(stderr, "-- \"%s\" base init: cleaning up unraised errors.\n", name);
	PyErr_Print();
	PyErr_Clear();
    }
    basefun = (struct BaseFunctions *) PyCObject_Import((char *)name, "functions");
    if (basefun) {
        if (PyErr_Occurred())
	    fprintf(stderr, "Error in \"%s\" import\n", name);
	Base_init(base, name, offset, basefun->simpleattr);
	return 1;
    } else {
	PyErr_Clear();
	return 0;
    }
}
