class StringConfigItem:
	def __init__(self, label, default = None, choosefrom = None,
	    help = '(mystery setting)', proc = None):
		self.label = label
		self.default = default
		self.choosefrom = choosefrom
		self.help = help
		self.proc = proc
		self.value = None

class BuiltinConfig(configure.Config):
	account = (... make a new one)
	ics_dfl = StringConfigItem('Character set assumed when reading article',
		default = 'iso-8859-1',
		choosefrom = ('iso-8859-1', 'utf-8'),
		help = '''
Character set encodings define the text character as binary
computer data.  PyNR knows how to deal with 2 different
character set encodings, ISO-8859-1 and UTF-8.  ISO-8859-1
is an 8 bit character set that works with European languages
(including English), and it's compatible with 7 bit US ASCII.
UTF-8 works for a broader range of languages, it's compatible
with US ASCII too, and it's the default for BeOS applications -
but not commonly used in USENET or email.

The input character set option is used only when the article
doesn't come with its own MIME header specifying character
set encoding.  The output character set is used for any new
article.
''')
	ocs_dfl = StringConfigItem('Character set assumed when posting article',
		default = 'iso-8859-1',
		choosefrom = ('iso-8859-1', 'utf-8'),
		help = ics_dfl.help)
	realname = StringConfigItem('Real World Identity',
		help = '''
Your real name.
''')
	default = (... what if there isn't any account yet?)
	quote = StringConfigItem('Prefix to quoted line', default = '> ')
	mimetypes = DictConfigItem('Match Applications to MIME types',
		columns = ('MIME type', 'file suffix', 'command template'),
		default = {'image/jpeg': ('.jpg', 'ShowImage %s'),
                'text/html': ('.html', 'NetPositive %s') })

class BuiltinAccountConfig(configure.Config):
	login = StringConfigItem('Login account name', help = '''
Your account name, for access to your email.
''')
	smtpserver = StringConfigItem('SMTP server', proc = validhost,
		help = '''
The host name to send mail to.  Commonly, this may be the same
host you go to, to read your mail.  Or a larger site might have
a special host just for this.  If you have no instructions to
the contrary, try your IMAP host.  If you choose an SMTP host
and your mail bounces from it with a complaint about "relaying",
you may be trying to mix service providers.
''')
	returnaddress = StringConfigItem('Email return address',
		proc = validemail, help = '''
Addresses to be considered aliases for yourself, when PyNR replies
to a message sent to them.  PyNR doesn't CC you on your own replies,
and this option allows you to apply that to other email addresses
under which you might receive mail, besides your PyNR return address.
''')


### (Move these into news and mail modules.)
class BuiltinNewsConfig(configure.Config):
	groups = ListConfigItem('USENET group', proc = groupconfig,
		default = ['comp.sys.be.help'])
	nntpserver = StringConfigItem('NNTP server', proc = validhost)

class BuiltinMailConfig(configure.Config):
	folders = ListConfigItem('Inbox IMAP remote folders', proc = folderconfig,
		default = ['INBOX'])
	readfolders = FolderListConfigItem('Other IMAP remote folders',
		proc = readfolderconfig, default = [])
	imapserver = ConfigItem('IMAP server', proc = validhost)
	imapcheckinterval = IntConfigItem('Minutes between INBOX checks',
		default = 3)
	authtypes = ListConfigItem('Authentication types',
		default = ['password'], choosefrom = ('password', 'Kerberos'))
	addresses = ListConfigItem('Your other addresses, not to be replied to',
		proc = validemail, default = [])

